@@ -637,11 +637,56 @@ int strhash(const char* str){
     g=h&0xF0000000;
     if(g) h^=g>>24;
     h&=0x0fffffff;
-    }
+  }
   GASSERT(h<=0x0fffffff);
   return h;
   }
 
+
+inline uint32_t fmix32 ( uint32_t h ) {
+  h ^= h >> 16;
+  h *= 0x85ebca6b;
+  h ^= h >> 13;
+  h *= 0xc2b2ae35;
+  h ^= h >> 16;
+  return h;
+}
+
+int murmur3(const char *key) {
+  int len=strlen(key);
+  const uint8_t * data = (const uint8_t*)key;
+  const int nblocks = len / 4;
+
+  uint32_t h1 = 5381; //seed;
+
+  const uint32_t c1 = 0xcc9e2d51;
+  const uint32_t c2 = 0x1b873593;
+
+  const uint32_t * blocks = (const uint32_t *)(data + nblocks*4);
+  for(int i = -nblocks; i; i++) {
+    uint32_t k1 = blocks[i];
+    k1 *= c1;
+    k1 = (k1 << 15) | (k1 >> (32 - 15));
+    k1 *= c2;
+    h1 ^= k1;
+    h1 = (k1 << 13) | (k1 >> (32 - 13));
+    h1 = h1*5+0xe6546b64;
+  }
+  const uint8_t * tail = (const uint8_t*)(data + nblocks*4);
+
+   uint32_t k1 = 0;
+   switch(len & 3)  {
+    case 3: k1 ^= tail[2] << 16;
+    case 2: k1 ^= tail[1] << 8;
+    case 1: k1 ^= tail[0];
+            k1 *= c1; k1 = (k1 << 15) | (k1 >> (32 - 15)); k1 *= c2; h1 ^= k1;
+   };
+
+    h1 ^= len;
+    return (fmix32(h1) & 0x7FFFFFFF);
+  }
+
+
 int djb_hash(const char* cp)
 {
     int h = 5381;
