@@ -49,10 +49,8 @@ extern bool debugMode;
 //collect all refguide transcripts for a single genomic sequence
 struct GRefData {
   GList<GffObj> rnas; //all transcripts on this genomic seq
-  //GList<GRefLocus> loci;
   int gseq_id;
   const char* gseq_name;
-   //GList<GTData> tdata; //transcript data (uptr holder for all rnas loaded here)
   GRefData(int gid=-1):rnas(false,true,false),gseq_id(gid),gseq_name(NULL) {
     gseq_id=gid;
     if (gseq_id>=0)
@@ -93,6 +91,48 @@ struct CBundlenode:public GSeg {
 			cov(_cov),bid(_bid),nextnode(_nextnode) {}
 };
 
+
+enum GPFType {
+	GPFT_NONE=0,
+	GPFT_TSS,
+	GPFT_CPAS
+}; //on 4 bits: maximum 15 types
+
+struct GPtFeature { //point feature (single coordinate)
+ GPFType ftype : 4;
+ int ref_id: 26; //index in a reftable[] with reference names, max 67,108,863
+ int strand: 2; //-1=-, 0=unstranded, +1=+
+ uint coord; //genomic coordinate for this feature
+ GPtFeature(GPFType ft=GPFT_NONE, int rid=-1, int _strand=0, uint loc=0):ftype(ft),
+		    ref_id(rid), strand(_strand), coord(loc) {}
+ bool operator<(const GPtFeature &o) { return coord<o.coord; }
+ bool operator==(const GPtFeature &o) { return coord==o.coord; }
+    //-- should really match ftype and strand too,
+    //   but we don't care, for bundle inclusion
+};
+
+
+struct GRefPtData {
+  int ref_id; //same with GPtFeature::ref_id, also in GffObj::names->gseqs
+  GList<GPtFeature> pfs; //all point feature on this genomic seq, sorted
+  GRefPtData(int gid=-1):ref_id(gid), pfs(true,false,false) { }
+
+  void add(GPtFeature* t) { //adds a fully formed GPtFeature record
+     if (ref_id!=t->ref_id || ref_id<0 || t->ref_id<0)
+           GError("Error: invalid call to GRefPtData::add() - cannot add feature with ref id %d to ref data id %d!\n",
+        		   t->ref_id, ref_id);
+     pfs.Add(t);
+  }
+  //sorting by unique ref_id
+  bool operator==(GRefPtData& d){
+    return ref_id==d.ref_id;
+  }
+  bool operator<(GRefPtData& d){
+    return (ref_id<d.ref_id);
+  }
+};
+
+
 // bundle data structure, holds all data needed for
 // infering transcripts from a bundle
 enum BundleStatus {
@@ -493,9 +533,11 @@ struct CGraphnode:public GSeg {
 	GBitVec childpat;
 	GBitVec parentpat;
 	GVec<int> trf; // transfrags that pass the node
+	bool hardstart:1; // verified/strong start
+	bool hardend:1;	// verified/strong end
 	//CGraphnode(int s=0,int e=0,unsigned int id=MAX_NODE,float nodecov=0,float cap=0,float r=0,float f=0):GSeg(s,e),nodeid(id),cov(nodecov),capacity(cap),rate(r),frag(f),child(),parent(),childpat(),parentpat(),trf(){}
 	CGraphnode(int s=0,int e=0,unsigned int id=MAX_NODE,float nodecov=0,float cap=0,float r=0):GSeg(s,e),
-			nodeid(id),cov(nodecov),capacity(cap),rate(r),child(),parent(),childpat(),parentpat(),trf(){}
+			nodeid(id),cov(nodecov),capacity(cap),rate(r),child(),parent(),childpat(),parentpat(),trf(),hardstart(false),hardend(false){}
 };
 
 // # 0: strand; 1: start; 2: end; 3: nreads; 4: nreads_good;
@@ -586,7 +628,7 @@ struct BundleData {
  int idx; //index in the main bundles array
  int start;
  int end;
- unsigned long numreads; // number of reads in bundles
+ unsigned long numreads; // number of reads in this bundle
  /*
  float wnumreads; // NEW: weighted numreads; a multi-mapped read mapped in 2 places will contribute only 0.5
  double sumreads; // sum of all reads' lengths in bundle
@@ -608,7 +650,7 @@ struct BundleData {
  GVec<float> bpcov[3];   // this needs to be changed to a more inteligent way of storing the data
  GList<CJunction> junction;
  GPVec<GffObj> keepguides;
- //GPVec<CTCov> covguides;
+ GPVec<GPtFeature> ptfs; //point features for this bundle
  GList<CPrediction> pred;
  RC_BundleData* rc_data;
  BundleData():status(BUNDLE_STATUS_CLEAR), idx(0), start(0), end(0),
@@ -616,7 +658,7 @@ struct BundleData {
 		 num_fragments(0), frag_len(0),sum_cov(0),covflags(0),
 		 refseq(), gseq(NULL), readlist(false,true), //bpcov(1024),
 		 junction(true, true, true),
-		 keepguides(false), pred(false), rc_data(NULL) {
+		 keepguides(false), ptfs(false), pred(false), rc_data(NULL) {
 	 for(int i=0;i<3;i++) 	bpcov[i].setCapacity(1024);
  }
 
@@ -655,6 +697,7 @@ struct BundleData {
 
  void Clear() {
 	keepguides.Clear();
+	ptfs.Clear();
 	pred.Clear();
 	pred.setSorted(false);
 	readlist.Clear();
