@@ -11,7 +11,7 @@
 #include "proc_mem.h"
 #endif
 
-#define VERSION "2.1.1"
+#define VERSION "2.1.2"
 
 //#define DEBUGPRINT 1
 
@@ -32,8 +32,8 @@
 #define USAGE "StringTie v" VERSION " usage:\n\
  stringtie <input.bam ..> [-G <guide_gff>] [-l <label>] [-o <out_gtf>] [-p <cpus>]\n\
   [-v] [-a <min_anchor_len>] [-m <min_tlen>] [-j <min_anchor_cov>] [-f <min_iso>]\n\
-  [-C <coverage_file_name>] [-c <min_bundle_cov>] [-g <bdist>] [-u] [-L]\n\
-  [-e] [-x <seqid,..>] [-A <gene_abund.out>] [-h] {-B | -b <dir_path>} \n\
+  [-C <coverage_file_name>] [-c <min_bundle_cov>] [-g <bdist>] [-u] [-L] [-e]\n\
+  [--ptf <f_tab>] [-x <seqid,..>] [-A <gene_abund.out>] [-h] {-B | -b <dir_path>} \n\
 Assemble RNA-Seq alignments into potential transcripts.\n\
  Options:\n\
  --version : print just the version at stdout and exit\n\
@@ -41,6 +41,7 @@ Assemble RNA-Seq alignments into potential transcripts.\n\
  --rf assume stranded library fr-firststrand\n\
  --fr assume stranded library fr-secondstrand\n\
  -G reference annotation to use for guiding the assembly process (GTF/GFF3)\n\
+ --ptf load point-features from a given 4 column feature file <f_tab>\n\
  -o output path/file name for the assembled transcripts GTF (default: stdout)\n\
  -l name prefix for output transcripts (default: STRG)\n\
  -f minimum isoform fraction (default: 0.01)\n\
@@ -171,7 +172,8 @@ GStr ballgown_dir;
 
 GFastaDb* gfasta=NULL;
 
-GStr guidegff;
+GStr guidegff; // -G option
+GStr ptff; // --ptf option (point features)
 
 bool debugMode=false;
 bool verbose=false;
@@ -192,10 +194,9 @@ double Frag_Len=0;
 double Cov_Sum=0;
 //bool firstPrint=true; //just for writing the GFF header before the first transcript is printed
 
-GffNames* gseqNames=NULL; //used as a dictionary for genomic sequence names and ids
-
-int refseqCount=0;
+GffNames* gseqNames=NULL; //used as a dictionary for reference sequence names and ids
 
+int refseqCount=0; // number of reference sequences found in the guides file
 
 #ifdef GMEMTRACE
  double maxMemRS=0;
@@ -242,6 +243,9 @@ bool moreBundles(); //thread-safe retrieves NoMoreBundles
 void noMoreBundles(); //sets NoMoreBundles to true
 //--
 void processOptions(GArgs& args);
+
+int loadPtFeatures(FILE* f, GArray<GRefPtData>& refpts);
+
 char* sprintTime();
 
 void processBundle(BundleData* bundle);
@@ -266,13 +270,16 @@ int main(int argc, char* argv[]) {
 
  // == Process arguments.
  GArgs args(argc, argv,
-   "debug;help;version;conservative;keeptmp;rseq=;bam;fr;rf;merge;exclude=zEihvteuLRx:n:j:s:D:G:C:S:l:m:o:a:j:c:f:p:g:P:M:Bb:A:F:T:");
+   "debug;help;version;conservative;keeptmp;rseq=;ptf=;bam;fr;rf;merge;"
+   "exclude=zEihvteuLRx:n:j:s:D:G:C:S:l:m:o:a:j:c:f:p:g:P:M:Bb:A:F:T:");
  args.printError(USAGE, true);
 
  processOptions(args);
 
  GVec<GRefData> refguides; // plain vector with transcripts for each chromosome
 
+ GArray<GRefPtData> refpts(true, true); // sorted,unique array of refseq point-features data
+
  //table indexes for Ballgown Raw Counts data (-B/-b option)
  GPVec<RC_TData> guides_RC_tdata(true); //raw count data or other info for all guide transcripts
  GPVec<RC_Feature> guides_RC_exons(true); //raw count data for all guide exons
@@ -360,6 +367,22 @@ const char* ERR_BAM_SORT="\nError: the input alignment file is not sorted!\n";
 	 }
  }
 
+ gseqNames=GffObj::names; //might have been populated already by gff data
+ gffnames_ref(gseqNames);  //initialize the names collection if not guided
+ bool havePtFeatures=false;
+
+ // -- loading point-feature data
+ if (!ptff.is_empty()) {
+   FILE* f=fopen(ptff.chars(),"r");
+   if (f==NULL) GError("Error: could not open reference annotation file (%s)!\n",
+       ptff.chars());
+   //                transcripts_only    sort by location?
+   int numptf=loadPtFeatures(f, refpts); //adds to gseqNames->gseqs accordingly, populates refpts
+   havePtFeatures=(numptf>0);
+   fclose(f);
+ }
+
+
 #ifdef GFF_DEBUG
   for (int r=0;r<refguides.Count();++r) {
 	  GRefData& grefdata = refguides[r];
@@ -372,18 +395,16 @@ const char* ERR_BAM_SORT="\nError: the input alignment file is not sorted!\n";
   exit(0);
 #endif
 
- // --- here we do the input processing
- gseqNames=GffObj::names; //might have been populated already by gff data
- gffnames_ref(gseqNames);  //initialize the names collection if not guided
+ // --- input processing
 
 
  GHash<int> hashread;      //read_name:pos:hit_index => readlist index
-
- GList<GffObj>* guides=NULL; //list of transcripts on a specific chromosome
-
+ GList<GffObj>* guides=NULL; //list of transcripts on a specific reference
+ GList<GPtFeature>* refptfs=NULL; //list of point-features on a specific reference
  int currentstart=0, currentend=0;
  int ng_start=0;
  int ng_end=-1;
+ int ptf_idx=0; //point-feature current index in the current (*refptfs)[]
  int ng=0;
  GStr lastref;
  bool no_ref_used=true;
@@ -534,10 +555,18 @@ if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
 	 }
 
 	 if (new_bundle || chr_changed) {
-		 //bgeneids.Clear();
 		 hashread.Clear();
 		 if (bundle->readlist.Count()>0) { // process reads in previous bundle
 			// (readthr, junctionthr, mintranscriptlen are globals)
+			if (refptfs) { //point-features defined for this reference
+				while (ptf_idx<refptfs->Count() && (int)(refptfs->Get(ptf_idx)->coord)<currentstart)
+					ptf_idx++;
+				//TODO: what if a PtFeature is nearby, just outside the bundle?
+				while (ptf_idx<refptfs->Count() && (int)(refptfs->Get(ptf_idx)->coord)<=currentend) {
+					bundle->ptfs.Add(refptfs->Get(ptf_idx)); //keep this PtFeature
+					ptf_idx++;
+				}
+			}
 			bundle->getReady(currentstart, currentend);
 			if (gfasta!=NULL) { //genomic sequence data requested
 				GFaSeqGet* faseq=gfasta->fetch(bundle->refseq.chars());
@@ -609,6 +638,17 @@ if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
 					 ng=guides->Count();
 				 }
 			 }
+			 if (havePtFeatures) {
+				 ptf_idx=-1;
+				 //setup refptf
+				 refptfs=NULL;
+				 GRefPtData rd(gseq_id);
+				 int ridx=refpts.IndexOf(rd);
+				 if (ridx>=0) {
+					refptfs=&(refpts[ridx].pfs);
+					ptf_idx=0;
+				 }
+			 }
 			 lastref=refseqName;
 			 lastref_id=gseq_id;
 			 currentend=0;
@@ -727,7 +767,7 @@ if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
 
  //cleaning up
  delete brec;
- //bamreader.bclose();
+
  bamreader.stop(); //close all BAM files
 
  if (guided && no_ref_used) {
@@ -759,6 +799,13 @@ if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
  if (mergeMode && guided )
 	 writeUnbundledGuides(refguides, f_out);
 
+
+ // clear refpts data, if loaded
+  if (refpts.Count()>0)
+	  for (int i=0;i<refpts.Count();i++) {
+		  refpts[i].pfs.setFreeItem(true);
+	  }
+
  fclose(f_out);
  if (c_out && c_out!=stdout) fclose(c_out);
 
@@ -1079,6 +1126,13 @@ void processOptions(GArgs& args) {
 	   else GError("Error: reference annotation file (%s) not found.\n",
 	             guidegff.chars());
 	 }
+	 s=args.getOpt("ptf");
+	 if (!s.is_empty()) {
+	   ptff=s;
+	   if (fileExists(ptff.chars())<=1)
+		   GError("Error: point features data file (%s) not found.\n",
+	             ptff.chars());
+	 }
 
 	 enableNames=(args.getOpt('E')!=NULL);
 
@@ -1317,6 +1371,7 @@ void processBundle(BundleData* bundle) {
 
 	}
 #endif
+
 	infer_transcripts(bundle);
 
 	if (ballgown && bundle->rc_data) {
@@ -1453,6 +1508,66 @@ int waitForData(BundleData* bundles) {
 
 #endif
 
+void addPtFeature(const char* refname, GPtFeature* pf, GArray<GRefPtData>& refpts) {
+  //expects gseqNames to be set to GffObj::names and initialized/populated already!
+  //MUST be called AFTER the guides file has been loaded (if given)
+  int gseq_id=gseqNames->gseqs.addName(refname);
+  pf->ref_id=gseq_id;
+  int ridx=-1;
+  GRefPtData* rpd=NULL;
+  GRefPtData rd(gseq_id);
+  if (refpts.Count()==0) {
+	  ridx=refpts.Add(rd);
+  } else {
+	  ridx=refpts.IndexOf(rd);
+	  if (ridx<0) {
+		  ridx=refpts.Add(rd);
+	  }
+  }
+  if (ridx<0) GError("Error adding GRefPtData entry (bug!)\n");
+  rpd = & refpts.Get(ridx);
+  rpd->add(pf);
+}
+
+int loadPtFeatures(FILE* f, GArray<GRefPtData>& refpts) {
+  //expected format:
+  //<chromosome> <coordinate> <strand> <feature_type>
+  int num=0;
+  GLineReader lr(f);
+  char* line=NULL;
+  GDynArray<char*> tokens;
+  while ((line=lr.nextLine())!=NULL) {
+    strsplit(line, tokens);
+    if (tokens.Count()<4)
+    	GError("Error parsing point-feature line (not enough columns):\n%s\n",line);
+    int start;
+    if (!strToInt(tokens[1], start))
+    	GError("Error parsing point-feature line (invalid coordinate):\n%s\n",line);
+    int8_t strand=-2;
+    if (strlen(tokens[2])==1) {
+    	if (tokens[2][0]=='+')
+    		strand=1;
+    	else if (tokens[2][0]=='-')
+    		strand=-1;
+    	else if (tokens[2][0]=='.')
+    		strand=0;
+    }
+    if (strand==-2)
+    	GError("Error parsing point-feature line (invalid strand):\n%s\n",line);
+    GPFType pftype=GPFT_NONE;
+    if (strcmp(tokens[3], "TSS")==0)
+			pftype=GPFT_TSS;
+	else if (strcmp(tokens[3], "CPAS")==0)
+			pftype=GPFT_CPAS;
+    if (pftype==0)
+    	GError("Error parsing point-feature line (unrecognized type):\n%s\n",line);
+    GPtFeature* ptf=new GPtFeature(pftype, -1, strand, start);
+    addPtFeature(tokens[0], ptf, refpts);
+    num++;
+  } //while line
+  return num;
+}
+
 void writeUnbundledGenes(GHash<CGene>& geneabs, const char* refseq, FILE* gout) {
 				 //write unbundled genes from this chromosome
 	geneabs.startIterate();
